/*

compile: gcc -O2 -o run oving3.c jpeg.c -static -ljpeg -lm
run ./run <bilde.jpg> <result.jpg>

*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "jpeg.h"

const PI2 = 6.283185;
double *costbl;
double *sintbl;
double *sintblh;
double *costblh;

//rbg pixel holder, with real and inmaginary parts
typedef struct {

  double reRed;
  double reGreen;
  double reBlue;
 
  double imRed;
  double imGreen;
  double imBlue;

}ccPixel;

//rbg pixel holder
typedef struct {
  unsigned char red;
  unsigned char green;
  unsigned char blue;
} cPixel;

//hsv pixel holder
typedef struct {
  int hue;
  int sat;
  int val;
} cHsv;

//bounding box
typedef struct {
  int start;
  int x_max;
  int x_min;
  int y_max;
  int y_min;
  int volume;
  double face_distance;
} cBox;

//transformed data
typedef struct {
	int width, height;
	double max,min;
	double *data;
} trans_image;

typedef struct {
	int count;
	cBox *regions;
} cbox_group;

typedef struct {
	int facenr;
	double *feature_vec;
} face;

int count_regions; //holds number of regions genereated by: generateRegions;
int c2_count;
int c3_count;
int c4_count;
int count_final  = 0;
int group_count = 0;

//Global constants
double facevec[37];

float DIST_TRESHOLD = 1.5;  //distance treshold to accept a face
int FEATURE_SIZE = 25;      //feature vector size
float ASPECT_MAX = 1.6;     //describes allowed aspect ratios
float ASPECT_MIN = 0.9;


/* saturated arithmetics; to avoid 255+1==0 and 0-1==255 */
uchar SAT(int val) {
	if      (val <   0) return 0;
	else if (val > 255) return 255;
	return (uchar) val;
}

int compare (const void* a, const void* b) {
  
  return (*(int*)a - *(int*)b );

}

void precalculateSinCos(int width, int height) {
  int i,w;
  costbl = (double*) malloc(sizeof(double)*width*width);
  sintbl = (double*) malloc(sizeof(double)*width*width);

  costblh = (double*) malloc(sizeof(double)*height*height);
  sintblh = (double*) malloc(sizeof(double)*height*height);

  for (w = 0; w < width;w++){
    for (i = 0;i < width;i++) {
    costbl[i+w*width]=cos((double)(PI2*w*i)/(double)(width));
    sintbl[i+w*width]=sin((double)(PI2*w*i)/(double)(width));
    }
  }

 for (w = 0; w < height;w++){
    for (i = 0;i < height;i++) {
    costblh[i+w*height]=cos((double)(PI2*w*i)/(double)(height));
    sintblh[i+w*height]=sin((double)(PI2*w*i)/(double)(height));
    }
  }



}



/* -----------------------------------
    Calculate HSV colors from RBG color
*/
cHsv hsv_from_rbg(cPixel rbg) {

  float r,b,g,temp;
  float max,min,delta;

  cHsv hsv;

  r = (float) (rbg.red)/255;
  b = (float) (rbg.blue)/255;
  g = (float) (rbg.green)/255;
 
  
  //find max and min
  if (r > b && r >  g) {
     max = r;
     if (b > g) {
       min = g;
     } else {
       min = b;
     }
  } else if (b > r && b > g) {
      max = b;
     if (r > g) {
       min = g;
     } else {
       min = r;
     }
  } else if (g > r && g > b) {
      max = g;
     if (b > r) {
       min = r;
     } else {
       min = b;
     }
  } else {
      max = r;
      min = r;
  }

  //check for black or white
  if (max == 0) {
    hsv.hue = 0; hsv.sat = 0; hsv.val = 0; 
    return hsv;
  } else if (max == min) {
    hsv.hue = 0; hsv.sat = 0; hsv.val =  max * 100 ;
    return hsv;
  }

  //normal color
  delta = max - min;
  //calculate hue
  if (r == max) {
    temp = (g - b) / delta;
  } else if (b == max) {
    temp = 2 + (b - r) / delta;
  } else {
    temp = 4 + (r - g) / delta;
  }
  temp = temp *  60;
 if (temp < 0) {temp +=360;}
 
hsv.hue = (int)(temp);
hsv.sat = (int)(100*(delta/max));
hsv.val = (int)(100*max);

return hsv;

}


/*--------------------------
   Skin color based treshold
*/
void treshSkin(imageRGB *img) {

  cPixel *pixelData = (cPixel*) img->data;

  int w,h,point;
  cHsv hsv;
  int keep;

  printf("Skin segmentation, HUE color based \n"); 

 for (h=0; h < img->height; h++) {
    for(w=0;w < img->width; w++) {
      point = (w + h*img->width);
      
      keep = 0;
      hsv = hsv_from_rbg(pixelData[point]); 

      if ((hsv.sat >= 10) && (hsv.val >= 40) && (hsv.sat <= (-hsv.hue - 0.1*hsv.val + 110)) && (hsv.hue <= (-0.4*hsv.val + 75)) ) {
	if (hsv.hue >= 0) {
	  if (hsv.sat <= (0.08*(100-hsv.val)*hsv.hue + 0.5*hsv.val)) {keep = 1;}
	} else {
	  if (hsv.sat <= 0.5*hsv.hue + 35) {keep = 1;}
	}
      }
     
     if (keep == 0) {
       pixelData[point].red = 0; pixelData[point].blue = 0; pixelData[point].green = 0;
     } else {
       // pixelData[point].red = 0; pixelData[point].blue = 0; pixelData[point].green = 255;
     }
   
  }
 }

}




int SAT5bit(int val) {
  double fraq  = (double)(val) / 256;
  fraq = fraq * 32;
  return((int)(fraq));
}

int *max_ind(int vec[32][32][32]) {
  int val = 0;

  int *ind = (int*) malloc (sizeof(int)*3);
  int i,h,w;
  ind[0]=ind[1]=ind[2]=0;

   for (h = 0; h < 32; h++) {
     for(i = 0; i < 32; i++) {
       for (w = 0; w < 32; w++) {
	  if ( (int)(vec[h][i][w]) > val) {  
	     val = (int)(vec[h][i][w]); 
             ind[0]=h;
             ind[1]=i;
             ind[2]=w;
	   }
       }
     }
  }

  return ind;
}

/*-----------------------
   Color Quantilization
*/
void quantColor(imageRGB *img,int colorsize,int sample) {
  

cPixel code_vec[colorsize];   //code vectors;
printf("\nGetting frequency, sampling step %d \n",sample); 


  //frequency 

  int rgbfreq[32][32][32];


  
  cPixel* pixeldata;
  pixeldata  = (cPixel*) img->data;
  
  int nwidth = img->width/sample;
  int nheight = img->height/sample;


  cPixel *newdata = (cPixel*) malloc (sizeof(cPixel)*(nwidth)*(nheight));

  int i,h,w;
  
  //assing to zero
   for (h = 0; h < 32; h++) {
     for(i = 0; i < 32; i++) {
       for (w = 0; w < 32; w++) {
       rgbfreq[h][w][i] = 0;
       }
     }
   }

   //calculate frequency
   for (h=0; h < nheight; h++) {
    for(w=0;w < nwidth; w++) {
      newdata[w + h*nwidth].red = pixeldata[w*sample + h*img->width*sample].red;
      newdata[w + h*nwidth].blue = pixeldata[w*sample + h*img->width*sample].blue;
      newdata[w + h*nwidth].green = pixeldata[w*sample + h*img->width*sample].green;

      rgbfreq[SAT5bit((int)newdata[w + h*nwidth].red)][SAT5bit((int)newdata[w + h*nwidth].green)][SAT5bit((int)newdata[w + h*nwidth].green)]++;

    }
   }

   free(newdata);
   rgbfreq[0][0][0] = 0; //dont use the color frequency of black

   printf("constructing palette, %d colors ",colorsize); 
   int colorfound = 0;
   int *bestind;
   int dist;
   int rd,gd,bd;
   int close;
   int reject = 0;

   while (colorfound < colorsize) {
     

     bestind = max_ind(rgbfreq);
     
     code_vec[colorfound].red   =  (int)((float)(bestind[0])/(float)(32)*255);
     code_vec[colorfound].green  =  (int)((float)(bestind[1])/(float)(32)*255);
     code_vec[colorfound].blue   = (int)((float)(bestind[2])/(float)(32)*255);

     rgbfreq[bestind[0]][bestind[1]][bestind[2]] = 0;

     close = 0;
      if (colorfound > 0 ) {
	for (i = 0; i < colorfound;i++) {

	  rd = (int)(code_vec[colorfound].red) - (int)(code_vec[i].red);
	  gd = (int)(code_vec[colorfound].green) - (int)(code_vec[i].green);
	  bd = (int)(code_vec[colorfound].blue) - (int)(code_vec[i].blue);

	  dist =  sqrt((rd*rd) + (gd*gd) + (bd*bd))  ;
	  
          if (dist < 35) {
	    close = 1;
          }

	}
      } 

      if (close == 0 || reject > 1000) { 
            printf("\n\t palette color %d (rgb): %d,%d,%d",colorfound,code_vec[colorfound].red,code_vec[colorfound].green,code_vec[colorfound].blue);
       colorfound++;
      } else {
	reject++;
      }

     free(bestind);
   }

 
    //run thorugh image and assign nearest color
    printf("\nmapping pixels to palette \n"); 
    int best_ind;
    float best_dist;

    for (i=0; i < img->height*img->width; i++) {

	 best_dist = -1;
        if (pixeldata[i].red != 0 && pixeldata[i].green != 0 && pixeldata[i].blue != 0) {
	 for (h = 0; h < colorsize; h++) {
	    dist = (pixeldata[i].red - code_vec[h].red)*(pixeldata[i].red - code_vec[h].red) + (pixeldata[i].blue - code_vec[h].blue)*(pixeldata[i].blue - code_vec[h].blue) + (pixeldata[i].green - code_vec[h].green)*(pixeldata[i].green - code_vec[h].green);
	     if (best_dist == -1 || dist < best_dist) {
	       best_dist = dist;
	       best_ind = h;
	     } //endif
	 } //next


	 pixeldata[i].red    = code_vec[best_ind].red;
	 pixeldata[i].blue   = code_vec[best_ind].blue;
	 pixeldata[i].green  = code_vec[best_ind].green;
        } //endif

      }
    

  
}



void tresh(cPixel* pixelData,int tval, int width, int height) {
 int w,h,p;

  printf("tresholding image,");
  
  float max_val;

for (w=0; w < width;w++) {
     for(h = 0; h < height;h++){
       p=w+h*width;
       
       max_val =  pixelData[p].red + pixelData[p].green  +  pixelData[p].blue;

       if ((max_val/3) > (float) (tval)) {
	 pixelData[p].red = 255;
	 pixelData[p].green = 255;
	 pixelData[p].blue = 255;
       } else {
	 pixelData[p].red = 1;
	 pixelData[p].green = 1;
	 pixelData[p].blue = 1;
       }


    }
  }

  printf("\tdone...\n");


}



void invertImage(cPixel* pixelData,int width, int height) {
  int w,h;
  printf("Inverting image,");
  
for (w=0; w < width;w++) {
     for(h = 0; h < height;h++){
       pixelData[w+h*width].red = 255 - pixelData[w+h*width].red;
       pixelData[w+h*width].green = 255 - pixelData[w+h*width].green;
       pixelData[w+h*width].blue = 255-pixelData[w+h*width].blue;
    }
  }

  printf("\tdone...\n");

}

void integralProjX(cPixel *pixelData, int width, int height) {
	printf("Integral projecting (x-axis), (img dim: %d,%d)",width,height);

	int w,h;
	long intval[width]; //array of sum of projected pixels
	long sum;  //sum of vertical pixels
	long hsum; //highest sum, used for scaling
	
	//do the prjection
	hsum = 0;
	sum = 0;

	for (w=0; w < width;w++) {
    	
  		 
		 for(h = 0; h < height;h++){
			sum += (long)( 255 - (pixelData[w+(h*width)].red+pixelData[w+(h*width)].green+pixelData[w+(h*width)].blue)/3);
			//printf(",%d",sum);
		 }

		 if (sum > hsum) {hsum = sum;}
		 intval[w] = sum;
		 sum = 0;
		// printf("\n width: (%d)  sum: %d",w,intval[w]);
	 }

	double dratio;
	//display the projection
	dratio = (double)hsum/height;
	
	for (w=0; w < width;w++) {

		 for(h = (height-1);0 <= h ;h--){
			
		 if ((height - h) > (intval[w]/dratio)) {
	
				pixelData[w+h*width].red = 255;
				pixelData[w+h*width].green = 255;
				pixelData[w+h*width].blue = 255;
			
			 }  else {
			 	pixelData[w+h*width].red = 1;
				pixelData[w+h*width].green = 1;
				pixelData[w+h*width].blue = 1;
			 }
		 }
	 }

	printf("\tdone...\n");
}

void grayScale(cPixel* pixelData,int width, int height) {
  int w,h;
  unsigned char newvalue;
  
  printf("Grayscaling image,");
  for (w=0; w < width;w++) {
     for(h = 0; h < height;h++){
       newvalue = (pixelData[w+h*width].red + pixelData[w+h*width].green + pixelData[w+h*width].blue)/3;
       pixelData[w+h*width].red = newvalue;
       pixelData[w+h*width].green = newvalue;
       pixelData[w+h*width].blue = newvalue;
    }
  }
  printf("\tdone...\n");
}

void adjustContrast(cPixel* pixelData, int amount, int width, int height) {
  int w,h;
  printf("Adjusting contrast,");
  
for (w=0; w < width;w++) {
     for(h = 0; h < height;h++){
       pixelData[w+h*width].red = SAT(pixelData[w+h*width].red+amount);
       pixelData[w+h*width].green = SAT(pixelData[w+h*width].green + amount);
       pixelData[w+h*width].blue = SAT(pixelData[w+h*width].blue + amount);
    }
  }

  printf("\tdone...\n");
}

/*
    -------------------------------------------------------------
	calculate the color distance between two rbg color points
*/
int rgb_dist(cPixel col1, cPixel col2) {
	int rr,gg,bb;
	int dist;
	rr = (col1.red - col2.red);
	gg = (col1.green - col2.green);
	bb = (col1.blue - col2.blue);

	dist = sqrt(rr*rr + gg*gg + bb*bb);

	return dist;
}


/*
  -------------------------------------------------------------
   Fills a region of the same color
   used recursive
*/
void fill(imageRGB *img, int point,cPixel targetcol,cBox *boundbox) {
  
  int coldist = 35;

	cPixel* pixeldata;
	pixeldata = (cPixel*) img->data;

       int y =  point/img->width;
       int x =  point - y * img->width;
       
       if (y > boundbox->y_max) { boundbox->y_max = y;}
       if (y < boundbox->y_min) { boundbox->y_min = y;}
       if (x > boundbox->x_max) { boundbox->x_max = x;}
       if (x < boundbox->x_min) { boundbox->x_min = x;}

       boundbox->volume++;
       // boundbox->hsv_color = hsv_from_rbg(pixeldata[point]);
       
       //targetcol = pixeldata[point];
       
       //dont include black as a color region
       if (pixeldata[point].red == 0 && pixeldata[point].green ==0 && pixeldata[point].blue ==0) {return;}

       pixeldata[point].red   = 0; 
       pixeldata[point].green = 0;
       pixeldata[point].blue  = 0;
	
	
       int new;

    //move up
       new = point - img->width;
       //if ((pixeldata[new].red == targetcol.red) && (pixeldata[new].green == targetcol.green) &&  (pixeldata[new].blue == targetcol.blue)) {
	if (rgb_dist(pixeldata[new],targetcol) < coldist) {
		   fill(img,new,targetcol,boundbox);
       }
     
  //move down
       new = point + img->width;
       //if ((pixeldata[new].red == targetcol.red) && (pixeldata[new].green == targetcol.green) &&  (pixeldata[new].blue == targetcol.blue)) {
	   if (rgb_dist(pixeldata[new],targetcol) < coldist) {      
			fill(img,new,targetcol,boundbox);
       }
    
   //move right
       new = point + 1;
       //if ((pixeldata[new].red == targetcol.red) && (pixeldata[new].green == targetcol.green) &&  (pixeldata[new].blue == targetcol.blue)) {
	   if (rgb_dist(pixeldata[new],targetcol) < coldist) {
          fill(img,new, targetcol,boundbox);
       }
     

  //move left
       new = point - 1;
       //if ((pixeldata[new].red == targetcol.red) && (pixeldata[new].green == targetcol.green) &&  (pixeldata[new].blue == targetcol.blue)) {
	 if (rgb_dist(pixeldata[new],targetcol) < coldist) {
			fill(img,new, targetcol,boundbox);
       }
    

}



/*
--------------------------------------------------------------
 Plots a region for visualisation on the input image
*/
void plot_box(imageRGB *img,cBox *box,cPixel color) {
	cPixel *pixeldata;
	pixeldata = (cPixel*) img->data;
    
	int w,h;


	if (box->volume == 0) {return;}


	for (h = box->y_min; h < box->y_max;h++) {
	  pixeldata[box->x_min + h*img->width].red = color.red;
	  pixeldata[box->x_min + h*img->width].green = color.green;
	  pixeldata[box->x_min + h*img->width].blue=color.blue;
	  
	  pixeldata[box->x_max + h*img->width].red = color.red;
	  pixeldata[box->x_max + h*img->width].green = color.green;
	  pixeldata[box->x_max + h*img->width].blue = color.blue;
	}

	for (w = box->x_min; w < box->x_max;w++) {

	  pixeldata[w + box->y_max*img->width].red = color.red;
	  pixeldata[w + box->y_max*img->width].green = color.green;
	  pixeldata[w + box->y_max*img->width].blue = color.blue;

	  pixeldata[w + box->y_min*img->width].red = color.red;
	  pixeldata[w + box->y_min*img->width].green = color.green;
	  pixeldata[w + box->y_min*img->width].blue = color.blue;

	}

	
}


/*
  -------------------------------------------------------------
  Generate regions of same colors
  Black (0,0,0) is treated as a non-color and will not make box
*/
cBox *generateRegion(imageRGB *img, imageRGB *drawon) {
	cPixel* pixeldata;
	pixeldata = (cPixel*) img->data;
	int count = 0;
	int point;
	cPixel black;
	black.green = black.blue = black.red = 0;

	cBox *boundbox = (cBox*) malloc(sizeof(cBox)*1000);

	int w,h;
        printf("\nCreating regions based on color\n");

       for (h=0; h < img->height;h++) {
	      for(w = 0; w < img->width;w++){
	     
		point = w+h*img->width;
		

	     if ( rgb_dist(pixeldata[point], black) > 30 ) {
	       boundbox[count].volume = 0;
	   
               int y =  point/img->width;
               int x =  point - y * img->width;
	       boundbox[count].x_max = boundbox[count].x_min = x;
	       boundbox[count].y_max = boundbox[count].y_min = y;
	       boundbox[count].start = point;
	       fill(img, point,pixeldata[point],&boundbox[count]);

	          if (boundbox[count].volume > 50) {
		        count++;
	            }
	     }

	   }
	 }

	 printf("\t\tbase regions: %d\n",count);
         count_regions = count;
 
	 // for (w=0; w < count;w++) {
	 //  plot_box(drawon,&boundbox[w]);
	 // }

	 return boundbox;

}


/*
  -------------------------------------------------------------
  merge two regions
*/
cBox merge_two(cBox *reg1,cBox *reg2,int i, int j) {
  cBox newbox;

  if (reg1[i].x_max > reg2[j].x_max) {
        newbox.x_max = reg1[i].x_max;
      } else {
	newbox.x_max = reg2[j].x_max;
  }
  if (reg1[i].x_min < reg2[j].x_min) {
        newbox.x_min = reg1[i].x_min;
      } else {
	newbox.x_min = reg2[j].x_min;
  }
  if (reg1[i].y_max > reg2[j].y_max) {
        newbox.y_max = reg1[i].y_max;
      } else {
	newbox.y_max = reg2[j].y_max;
  }
  if (reg1[i].y_min < reg2[j].y_min) {
        newbox.y_min = reg1[i].y_min;
      } else {
	newbox.y_min = reg2[j].y_min;
  }

  //newbox.hsv_color.hue = (reg[i].hsv_color.hue + reg[j].hsv_color.hue)/2;
  //newbox.hsv_color.sat = (reg[i].hsv_color.sat + reg[j].hsv_color.sat)/2;
  //newbox.hsv_color.val = (reg[i].hsv_color.val + reg[j].hsv_color.val)/2;

  newbox.volume = reg1[i].volume + reg2[j].volume;
 
    return newbox;
}

/*
  -------------------------------------------------------------
  merge two regions
*/
cBox merge(cBox *reg,int i, int j) {
  cBox newbox;

  if (reg[i].x_max > reg[j].x_max) {
        newbox.x_max = reg[i].x_max;
      } else {
	newbox.x_max = reg[j].x_max;
  }
  if (reg[i].x_min < reg[j].x_min) {
        newbox.x_min = reg[i].x_min;
      } else {
	newbox.x_min = reg[j].x_min;
  }
  if (reg[i].y_max > reg[j].y_max) {
        newbox.y_max = reg[i].y_max;
      } else {
	newbox.y_max = reg[j].y_max;
  }
  if (reg[i].y_min < reg[j].y_min) {
        newbox.y_min = reg[i].y_min;
      } else {
	newbox.y_min = reg[j].y_min;
  }

  //newbox.hsv_color.hue = (reg[i].hsv_color.hue + reg[j].hsv_color.hue)/2;
  //newbox.hsv_color.sat = (reg[i].hsv_color.sat + reg[j].hsv_color.sat)/2;
  //newbox.hsv_color.val = (reg[i].hsv_color.val + reg[j].hsv_color.val)/2;

  newbox.volume = reg[i].volume + reg[j].volume;
 
    return newbox;
}


/*
  -------------------------------------------------------------
  merge regions which are similar based on colordistance and position
*/
cBox *merge_regions(cBox *reg1,cBox *reg2,int rc1,int rc2,int *counter) {
  printf("\nMerging regions");
  cBox *newreg = (cBox*) malloc(sizeof(cBox)*10000);
  int i;
  int ind;
  int new_count = 0;
  int coldist,hh,ss,vv;

  ind = 0;
  for (ind = 0; ind < rc1; ind++) {
  for (i = 0; i < rc2; i++) {


      //merge om overlap i region
      if ((reg1[ind].x_min > reg2[i].x_min && reg1[ind].x_min < reg2[i].x_max) || (reg2[i].x_min > reg1[ind].x_min && reg2[i].x_min < reg1[ind].x_max)) {
        if ((reg1[ind].y_min > reg2[i].y_min && reg1[ind].y_min < reg2[i].y_max) || (reg2[i].y_min > reg1[ind].y_min && reg2[i].y_min < reg1[ind].y_max)) {
			newreg[new_count] = merge_two(reg1,reg2,ind,i);
			new_count++;
		}
      } 

      //merge om overlap i x og y er nre
       
    if ((reg1[ind].x_min > reg2[i].x_min && reg1[ind].x_min < reg2[i].x_max) || (reg2[i].x_min > reg1[ind].x_min && reg2[i].x_min < reg1[ind].x_max)) {
		if ( (abs(reg1[ind].y_min - reg2[i].y_max) < 25) || (abs(reg2[i].y_min - reg1[ind].y_max) < 25) )   {
			newreg[new_count] = merge_two(reg1,reg2,ind,i);
			new_count++;
		}
    }
    


   }
  }

  *counter = new_count;
  printf("\n\t\tnew (merged) regions: %d",*counter);

  return newreg;

}



/*
  -------------------------------------------------------------
  merge regions which are similar based on colordistance and position
*/
cBox *mergeRegions(cBox *reg,int *counter) {
  printf("\nMerging regions");
  cBox *newreg = (cBox*) malloc(sizeof(cBox)*10000);
  int i;
  int ind;
  int new_count = 0;
  int coldist,hh,ss,vv;

  ind = 0;
  for (ind = 0; ind < count_regions; ind++) {
  for (i = ind; i < count_regions; i++) {
    if (ind != i) {

      
      //merge om overlap i region
      if ((reg[ind].x_min > reg[i].x_min && reg[ind].x_min < reg[i].x_max) || (reg[i].x_min > reg[ind].x_min && reg[i].x_min < reg[ind].x_max)) {
        if ((reg[ind].y_min > reg[i].y_min && reg[ind].y_min < reg[i].y_max) || (reg[i].y_min > reg[ind].y_min && reg[i].y_min < reg[ind].y_max)) {
			newreg[new_count] = merge(reg,ind,i);
			new_count++;
		}
      } 

      //merge om overlap i x og y er nre
       
    if ((reg[ind].x_min > reg[i].x_min && reg[ind].x_min < reg[i].x_max) || (reg[i].x_min > reg[ind].x_min && reg[i].x_min < reg[ind].x_max)) {
	 if ( (abs(reg[ind].y_min - reg[i].y_max) < 25) || (abs(reg[i].y_min - reg[ind].y_max) < 25) )   {
		newreg[new_count] = merge(reg,ind,i);
		new_count++;
	}

      }
    
    }

   }
  }

  *counter = new_count;
  printf("\n\t\tnew (merged) regions: %d",*counter);

  return newreg;

}

/*
  -------------------------------------------------------------
  return 1 if region is uniqe or 0 if region exists
  input: collection of regions, region  to add
*/
int add_region(cBox *regions, cBox new_region) {
  int w;
  float aspect = 0;

  aspect =  (float) (new_region.y_max - new_region.y_min) / (float) (new_region.x_max - new_region.x_min);
 
  if (aspect < ASPECT_MIN || aspect > ASPECT_MAX || new_region.volume < 200) {
    return 0;
  }

  for (w = 0; w < count_final;w++) {
    if (regions[w].x_min == new_region.x_min && regions[w].x_max == new_region.x_max && regions[w].y_min == new_region.y_min && regions[w].y_max == new_region.y_max) {
      return 0;
    }
  }
 

	 regions[count_final].x_min = new_region.x_min; regions[count_final].x_max = new_region.x_max;
	 regions[count_final].y_min = new_region.y_min; regions[count_final].y_max = new_region.y_max;
	 regions[count_final].volume = new_region.volume;

  count_final++;
  
  return 1;
}


/*
  -------------------------------------------------------------
  Generate image data out of region
*/
imageRGB image_from_region(cBox region,imageRGB *original) {
  
  imageRGB retimg;
  int w,h;
  retimg.width = region.x_max - region.x_min;
  retimg.height = region.y_max - region.y_min;
  int ind1, ind2;
  // printf("\n \timage region (width, height): %d, %d ",retimg.width,retimg.height);
 
  cPixel *imgdata = (cPixel*) malloc(sizeof(cPixel)*retimg.width*retimg.height);
  cPixel *org = (cPixel*) original->data;

  for (h = 0; h < retimg.height; h++) {
    for (w = 0; w < retimg.width; w++) {
      
      ind1 = w + h*retimg.width;
      ind2 = (region.x_min + w) + (region.y_min+h)*original->width;

      imgdata[ind1].red = org[ind2].red;
      imgdata[ind1].green = org[ind2].green;
      imgdata[ind1].blue = org[ind2].blue;

    }
  }
 
  retimg.data = (uchar*) imgdata;

  return retimg;
}

/*
  -------------------------------------------------------------
  Haar wavelet transform
*/
trans_image har_avg(imageRGB *img,int type, trans_image *timg, int usetimg) {
	cPixel* pixeldata;
	int height,width;
	trans_image retimg;

	if (usetimg == 0) {
		pixeldata = (cPixel*) img->data;
		retimg.width = img->width/2;
		retimg.height = img->height/2;
		width  = img->width;
		height = img->height;
	} else {
		retimg.width = timg->width/2;
		retimg.height = timg->height/2;
		width  = timg->width; 
		height = timg->height;
	}
	

	double *avgtmp = (double*) malloc(sizeof(double)*(width)*(height));
	retimg.max = retimg.min = 0;
    double *retdata = (double*) malloc(sizeof(double)*(retimg.width*retimg.height));
	
	int int1,int2;
	double d1,d2;
    int i,j;
 
   
//Rows

	for (j = 0; j < (height); j++)  {

		for(i = 0; i < (retimg.width); i++) {
			if (usetimg == 0) {
				int1 = (pixeldata[(i*2) + j*(width)].red + pixeldata[(i*2) + j*width].green + pixeldata[(i*2) + j*width].blue)/3;
				int2 = (pixeldata[(i*2+1) + j*(width)].red + pixeldata[(i*2+1) + j*width].green + pixeldata[(i*2+1) + j*width].blue)/3;
			}

			if (type == 1 || type==2) {
				if (usetimg == 0) {
					avgtmp[i+j*retimg.width] = ((double)(int1) + (double)(int2)) / sqrt(2);
				} else {
					avgtmp[i+j*retimg.width] = (timg->data[(i*2) + j*(width)] + timg->data[(i*2+1) + j*(width)]) / sqrt(2);
				}
			} else {
				if (usetimg == 0) {
					avgtmp[i+j*retimg.width] = ((double)(int1) - (double)(int2)) / sqrt(2);
				} else {
				    avgtmp[i+j*retimg.width] = (timg->data[(i*2) + j*(width)] - timg->data[(i*2+1) + j*(width)]) / sqrt(2);
				}
			}


		}
	}

//colums
		for(i = 0; i < retimg.width; i++) {
			for (j = 0; j < retimg.height; j++)  {
			 
			//if (print == 1) {printf("\n workvalues: (index: %d, %d): %f AND %f",(i) + j*retimg.width*2,(i)+(j+1)*retimg.width*2,avgtmp[(i) + j*retimg.width*2],avgtmp[(i)+(j+1)*retimg.width*2]); }

			d1 = avgtmp[(i) + j*retimg.width*2];
			d2 = avgtmp[(i)+(j+1)*retimg.width*2];

			if (!(d2 < 100000) || !(d2 > -100000)) {d2 = 0;}

			if (type == 1 || type==3) {
				avgtmp[i+j*retimg.width]= (d1 + d2) / sqrt(2);
			} else {
				avgtmp[i+j*retimg.width]= (d1 - d2) / sqrt(2);
			}

			if (avgtmp[i+j*retimg.width] > retimg.max) {retimg.max = avgtmp[i+j*retimg.width];}	
			if (avgtmp[i+j*retimg.width] < retimg.min) {retimg.min = avgtmp[i+j*retimg.width];}
	
			//if (print == 1) {printf("\n transformed value:: %f",avgtmp[i+j*retimg.width]);}
			retdata[i+j*retimg.width] = avgtmp[i+j*retimg.width];
			  
			}
		}

		//if (print == 1) {printf("\n transformed max/min: %f/%f",retimg.max,retimg.min); }
		//if (print == 1) {printf("\n org image dimension %d %d, new %d %d",img->width,img->height,retimg.width,retimg.height); }

		free(avgtmp);
		retimg.data = retdata;

		return retimg;
				
}

/*
  -------------------------------------------------------------
  Writes transformed double values as a image, values will be scaled between 0-255
*/

void write_transformed_JPG(char *file_out,int quality,trans_image *img) {
 //scale between 0-255
 int i;
 imageRGB img_out;
 img_out.width = img->width;
 img_out.height = img->height;
 cPixel *pixeldata = (cPixel*) malloc(sizeof(cPixel)*img->width*img->height);
 
 for (i = 0; i < img->width*img->height;i++) {
		 pixeldata[i].red = pixeldata[i].green = pixeldata[i].blue = SAT((int)(((img->data[i] + abs(img->min))/(abs(img->min) + img->max))*255));
		// printf("\t %d",pixeldata[i].red);
 }
 img_out.data = (uchar*) pixeldata;
 write_JPEG_file (file_out, quality, img_out);
 //write
 free(img_out.data);

}


/*
  -------------------------------------------------------------
  Calculates the mean of a double vector, n must be specified
*/
double mean(double *vals, int n) {
int i;
double mean = 0;

 for (i = 0; i < n; i++) {
	 mean += vals[i];
 }

return (mean/n);
}


/*
  -------------------------------------------------------------
  Calculate the varianse of double vector, mean and n is required as input
*/
double var(double *vals, double mean, int n) {
int i;
double var = 0;
 
 for (i = 0; i < n; i++) {
	var += (vals[i]-mean)*(vals[i]-mean);
 }

 return (var/n);
}

/*
 ------------------------------------------------------------
  the Bhattacharrya distance
  input: featurevector
*/
double dist_bhatta(double *featurevec) {
  int i;
  int n = FEATURE_SIZE;
  double dist = 0;

  for (i = 0; i < n; i++) {
       
	 // if (facevec[i] == 0) {facevec[i] = 0.5;}
	 // printf("\n\t\t\t feature val: %f, face val: %f",featurevec[i],facevec[i]);

	  dist += log(  (featurevec[i] + facevec[i]) / (2*sqrt(featurevec[i]*facevec[i])) );
  }
	
  return dist;
}
/*
 ------------------------------------------------------------
  the Bhattacharrya distance
  input: featurevector
*/
double dist_bhatta2(double *vec1, double *vec2) {
  int i;
  int n = FEATURE_SIZE;
  double dist = 0;

  for (i = 0; i < n; i++) {
	  dist += log(  (vec1[i] + vec2[i]) / (2*sqrt(vec1[i]*vec2[i])) );
  }
	
  return dist;
}


double dist_leastsq2(double *vec1, double *vec2) {
  int i;
  int n = FEATURE_SIZE;
  double dist = 0;


  for (i = 0; i < n; i++) {
    dist += (vec1[i] - vec2[i])*(vec1[i] - vec2[i]);
  }

  return sqrt(dist);
}

/*
 ------------------------------------------------------------
  the least square distance
  input: featurevector
*/
double dist_leastsq(double *featurevec) {
  int i;
  int n = FEATURE_SIZE;
  double dist = 0;


  for (i = 0; i < n; i++) {

    //printf("\n\t\t\t feature val: %f, face val: %f",featurevec[i],facevec[i]);

    dist += (featurevec[i] - facevec[i])*(featurevec[i] - facevec[i]);
  }


  return dist;
}


/*
 ------------------------------------------------------------
  calculate variance in a region of transformed image
*/
double var_in_region(double *source_data,cBox data_region) {
	int w,h;
	double avg,var;
	int height,width;
	width = data_region.x_max - data_region.x_min;
	height = data_region.y_max - data_region.y_min;
	avg = var = 0;

	for (h = data_region.y_min; h < data_region.y_max;h++) {
	  	for (w = data_region.x_min; w < data_region.x_max;w++) {
		  avg += source_data[w + width*h];
		}
	}
	avg = (avg/(width*height));

	for (h = data_region.y_min; h < data_region.y_max;h++) {
        		for (w = data_region.x_min; w < data_region.x_max;w++) {
			 var += (source_data[w +  width*h] - avg)*(source_data[w + width*h] - avg);
		}
	}
	return (var/(width*height));
}


/*
 ------------------------------------------------------------
 Generates the feautre vector from a region
 currently a 2 scale, with hardcoded splits
*/
void feature_extract_tree(imageRGB *img_reg, double *feature_vec, int scale){
	trans_image haar[16];
	trans_image org[4];
	cBox split[4];
	double m;
	char str[50];
	int i;
	org[0] = har_avg(img_reg,1,0,0);
    org[1] = har_avg(img_reg,2,0,0);
    org[2] = har_avg(img_reg,3,0,0);
    org[3] = har_avg(img_reg,4,0,0);


	split[0].x_min = 0;		split[0].x_max = (int)(org[0].width);
	split[0].y_min = 0;		split[0].y_max = (int)(org[0].height);

	m = mean(org[0].data,org[0].width*org[0].height);

	feature_vec[0] = var(org[0].data, m, org[0].width*org[0].height);
	
	feature_vec[1] = var_in_region(org[1].data,split[0]);

	feature_vec[2] = var_in_region(org[2].data,split[0]);

	feature_vec[3] = var_in_region(org[3].data,split[0]);


	//next levlel

		haar[0] = har_avg(img_reg,1,&org[0],1);
		haar[1] = har_avg(img_reg,2,&org[0],2);
		haar[2] = har_avg(img_reg,3,&org[0],3);
		haar[3] = har_avg(img_reg,4,&org[0],4);

		haar[4] = har_avg(img_reg,1,&org[1],1);
		haar[5] = har_avg(img_reg,2,&org[1],2);
		haar[6] = har_avg(img_reg,3,&org[1],3);
		haar[7] = har_avg(img_reg,4,&org[1],4);

		haar[8] = har_avg(img_reg,1,&org[2],1);
		haar[9] = har_avg(img_reg,2,&org[2],2);
		haar[10] = har_avg(img_reg,3,&org[2],3);
		haar[11] = har_avg(img_reg,4,&org[2],4);

		haar[12] = har_avg(img_reg,1,&org[3],1);
		haar[13] = har_avg(img_reg,2,&org[3],2);
		haar[14] = har_avg(img_reg,3,&org[3],3);
		haar[15] = har_avg(img_reg,4,&org[3],4);

		split[0].x_min = 0;		split[0].x_max = (int)(haar[0].width);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[0].height);


	 feature_vec[4] = var_in_region(haar[0].data,split[0]);
	 feature_vec[5] = var_in_region(haar[1].data,split[0]);
	 feature_vec[6] = var_in_region(haar[2].data,split[0]);
	 feature_vec[7] = var_in_region(haar[3].data,split[0]);

	 feature_vec[8] = var_in_region(haar[4].data,split[0]);
	 feature_vec[9] = var_in_region(haar[5].data,split[0]);
	 feature_vec[10] = var_in_region(haar[6].data,split[0]);
	 feature_vec[11] = var_in_region(haar[7].data,split[0]);
	 
	 feature_vec[12] = var_in_region(haar[8].data,split[0]);
	 feature_vec[13] = var_in_region(haar[9].data,split[0]);
	 feature_vec[14] = var_in_region(haar[10].data,split[0]);
	 feature_vec[15] = var_in_region(haar[11].data,split[0]);

	 feature_vec[16] = var_in_region(haar[12].data,split[0]);
	 feature_vec[17] = var_in_region(haar[13].data,split[0]);
	 feature_vec[18] = var_in_region(haar[14].data,split[0]);
	 feature_vec[19] = var_in_region(haar[15].data,split[0]);

	 for (i = 0; i < 16; i++) {
		 sprintf(str,"haar%d.jpg",i);
		// write_transformed_JPG(str,75,&haar[i]);

		 if (i < 4) {
			 free(org[i].data);
		 }
		
		 free(haar[i].data);
	 }

}



/*
 ------------------------------------------------------------
 Generates the feautre vector from a region
 currently a 2 scale, with hardcoded splits
*/
void feature_extract(imageRGB *img_reg, double *feature_vec){
	int scale = 2;
	int i;
	trans_image haar[2+scale*3];
	cBox split[4];
	double m;

	haar[0] = har_avg(img_reg,1,0,0);
	
    haar[1] = har_avg(img_reg,2,0,0);
    haar[2] = har_avg(img_reg,3,0,0);
    haar[3] = har_avg(img_reg,4,0,0);
	
	//next levlel
	//haar[4] = har_avg(img_reg,1,&haar[0],1);
	haar[5] = har_avg(img_reg,2,&haar[0],1);
	haar[6] = har_avg(img_reg,3,&haar[0],1);
	haar[7] = har_avg(img_reg,4,&haar[0],1);


		//top left
		split[0].x_min = 0;		split[0].x_max = (int)(haar[0].width/2);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[0].height/2);
		//top right
		split[1].x_min = (int)(haar[0].width/2);	split[1].x_max = haar[0].width;
		split[1].y_min = 0;		                split[1].y_max = (int)(haar[0].height/2);
		//bottom left
		split[2].x_min = 0;			        split[2].x_max =  (int)(haar[0].width/2);
		split[2].y_min = (int)(haar[0].height/2);	split[2].y_max = haar[0].height;
		//bottom right
		split[3].x_min = (int)(haar[0].width/2);;	split[3].x_max = haar[0].width;
		split[3].y_min = (int)(haar[0].height/2);;	split[3].y_max = haar[0].height;


	 m = mean(haar[0].data,haar[0].width*haar[0].height);
	 feature_vec[0] = var(haar[0].data, m, haar[0].width*haar[0].height);

	 feature_vec[1] = var_in_region(haar[1].data,split[0]);
	 feature_vec[2] = var_in_region(haar[1].data,split[1]);
	 feature_vec[3] = var_in_region(haar[1].data,split[2]);
	 feature_vec[4] = var_in_region(haar[1].data,split[3]);

	 feature_vec[5] = var_in_region(haar[2].data,split[0]);
	 feature_vec[6] = var_in_region(haar[2].data,split[1]);
	 feature_vec[7] = var_in_region(haar[2].data,split[2]);
	 feature_vec[8] = var_in_region(haar[2].data,split[3]);

	 feature_vec[9] = var_in_region(haar[3].data,split[0]);
	 feature_vec[10] = var_in_region(haar[3].data,split[1]);
	 feature_vec[11] = var_in_region(haar[3].data,split[2]);
	 feature_vec[12] = var_in_region(haar[3].data,split[3]);


		//top left
		split[0].x_min = 0;		split[0].x_max = (int)(haar[5].width/2);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[5].height/2);
		//top right
		split[1].x_min = (int)(haar[5].width/2);	split[1].x_max = haar[5].width;
		split[1].y_min = 0;		                split[1].y_max = (int)(haar[5].height/2);
		//bottom left
		split[2].x_min = 0;			        split[2].x_max =  (int)(haar[5].width/2);
		split[2].y_min = (int)(haar[5].height/2);	split[2].y_max = haar[5].height;
		//bottom right
		split[3].x_min = (int)(haar[5].width/2);;	split[3].x_max = haar[5].width;
		split[3].y_min = (int)(haar[5].height/2);;	split[3].y_max = haar[5].height;

	
	 feature_vec[13] = var_in_region(haar[5].data,split[0]);
	 feature_vec[14] = var_in_region(haar[5].data,split[1]);
	 feature_vec[15] = var_in_region(haar[5].data,split[2]);
	 feature_vec[16] = var_in_region(haar[5].data,split[3]);

	 feature_vec[17] = var_in_region(haar[6].data,split[0]);
	 feature_vec[18] = var_in_region(haar[6].data,split[1]);
	 feature_vec[19] = var_in_region(haar[6].data,split[2]);
	 feature_vec[20] = var_in_region(haar[6].data,split[3]);

	 feature_vec[21] = var_in_region(haar[7].data,split[0]);
	 feature_vec[22] = var_in_region(haar[7].data,split[1]);
	 feature_vec[23] = var_in_region(haar[7].data,split[2]);
	 feature_vec[24] = var_in_region(haar[7].data,split[3]);

	 /*
	 write_transformed_JPG("har10.jpg",75,&haar[0]);
	 write_transformed_JPG("har11.jpg",75,&haar[1]);
	 write_transformed_JPG("har12.jpg",75,&haar[2]);
	 write_transformed_JPG("har13.jpg",75,&haar[3]);
	 //	 write_transformed_JPG("har20.jpg",75,&haar[4]);
	 write_transformed_JPG("har21.jpg",75,&haar[5]);
	 write_transformed_JPG("har22.jpg",75,&haar[6]);
	 write_transformed_JPG("har23.jpg",75,&haar[7]);
	 */
	 //printf("\nhei %f %f",feature_vec[0],feature_vec[24]);

	free(haar[0].data);
    free(haar[1].data);
    free(haar[2].data);
    free(haar[3].data);
	free(haar[4].data);
	free(haar[5].data);
	free(haar[6].data);
	free(haar[7].data);
}



/*
 ------------------------------------------------------------
 Generates the feautre vector from a region
 currently a 2 scale, with hardcoded splits
*/
void feature_extractC(imageRGB *img_reg, double *feature_vec, int scale){
	trans_image haar[5];
	cBox split[4];
	double m;
	char str[50];
	int i;
	haar[0] = har_avg(img_reg,1,0,0);
    haar[1] = har_avg(img_reg,2,0,0);
    haar[2] = har_avg(img_reg,3,0,0);
    haar[3] = har_avg(img_reg,4,0,0);
	
	haar[4] = har_avg(img_reg,1,0,0);

	split[0].x_min = 0;		split[0].x_max = (int)(haar[4].width/2);
	split[0].y_min = 0;		split[0].y_max = (int)(haar[4].height/2);

	split[1].x_min = (int)(haar[4].width/2);		split[1].x_max = (int)(haar[4].width);
	split[1].y_min = 0;								split[1].y_max = (int)(haar[4].height/2);

	split[2].x_min = 0;								split[2].x_max = (int)(haar[4].width/2);
	split[2].y_min = (int)(haar[4].height/2);		split[2].y_max = (int)(haar[4].height);

	split[3].x_min = (int)(haar[4].width/2);		split[3].x_max = (int)(haar[4].width);
	split[3].y_min = (int)(haar[4].height/2);		split[3].y_max = (int)(haar[4].height);

	m = mean(haar[4].data,haar[4].width*haar[4].height);
	feature_vec[0] = var(haar[0].data, m, haar[0].width*haar[0].height);
	
	feature_vec[1] = var_in_region(haar[1].data,split[0]);
	feature_vec[2] = var_in_region(haar[1].data,split[1]);
	feature_vec[3] = var_in_region(haar[1].data,split[2]);
	feature_vec[4] = var_in_region(haar[1].data,split[3]);

	feature_vec[5] = var_in_region(haar[2].data,split[0]);
	feature_vec[6] = var_in_region(haar[2].data,split[1]);
	feature_vec[7] = var_in_region(haar[2].data,split[2]);
	feature_vec[8] = var_in_region(haar[2].data,split[3]);

	feature_vec[9] = var_in_region(haar[3].data,split[0]);
	feature_vec[10] = var_in_region(haar[3].data,split[1]);
	feature_vec[11] = var_in_region(haar[3].data,split[2]);
	feature_vec[12] = var_in_region(haar[3].data,split[3]);

	//next levlel
	//haar[4] = har_avg(img_reg,1,&haar[0],1);
	for (i = 1; i < scale; i++) {
	
		haar[1] = har_avg(img_reg,2,&haar[4],1);
		haar[2] = har_avg(img_reg,3,&haar[4],1);
		haar[3] = har_avg(img_reg,4,&haar[4],1);
	
		haar[4] = har_avg(img_reg,1,&haar[4],1);

		split[0].x_min = 0;		split[0].x_max = (int)(haar[4].width/2);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[4].height/2);

		split[1].x_min = (int)(haar[4].width/2);		split[1].x_max = (int)(haar[4].width);
		split[1].y_min = 0;								split[1].y_max = (int)(haar[4].height/2);

		split[2].x_min = 0;								split[2].x_max = (int)(haar[4].width/2);
		split[2].y_min = (int)(haar[4].height/2);		split[2].y_max = (int)(haar[4].height);

		split[3].x_min = (int)(haar[4].width/2);		split[3].x_max = (int)(haar[4].width);
		split[3].y_min = (int)(haar[4].height/2);		split[3].y_max = (int)(haar[4].height);

	

	 feature_vec[13 + ((i-1)*12)] = var_in_region(haar[1].data,split[0]);
	 feature_vec[14 + ((i-1)*12)] = var_in_region(haar[1].data,split[1]);
	 feature_vec[15 + ((i-1)*12)] = var_in_region(haar[1].data,split[2]);
	 feature_vec[16 + ((i-1)*12)] = var_in_region(haar[1].data,split[3]);

	 feature_vec[17 + ((i-1)*12)] = var_in_region(haar[2].data,split[0]);
	 feature_vec[18 + ((i-1)*12)] = var_in_region(haar[2].data,split[1]);
	 feature_vec[19 + ((i-1)*12)] = var_in_region(haar[2].data,split[2]);
	 feature_vec[20 + ((i-1)*12)] = var_in_region(haar[2].data,split[3]);
	 
	 feature_vec[21 + ((i-1)*12)] = var_in_region(haar[3].data,split[0]);
	 feature_vec[22 + ((i-1)*12)] = var_in_region(haar[3].data,split[1]);
	 feature_vec[23 + ((i-1)*12)] = var_in_region(haar[3].data,split[2]);
	 feature_vec[24 + ((i-1)*12)] = var_in_region(haar[3].data,split[3]);

	 
	free(haar[0].data);
    free(haar[1].data);
    free(haar[2].data);
    free(haar[3].data);

	}

}




/*
 ------------------------------------------------------------
 Generates the feautre vector from a region
 currently a 2 scale, with hardcoded splits
*/
void feature_extractB(imageRGB *img_reg, double *feature_vec, int scale){
	trans_image haar[5];
	cBox split[4];
	double m;
	char str[50];
	int i;
	haar[0] = har_avg(img_reg,1,0,0);
    haar[1] = har_avg(img_reg,2,0,0);
    haar[2] = har_avg(img_reg,3,0,0);
    haar[3] = har_avg(img_reg,4,0,0);
	
	haar[4] = har_avg(img_reg,1,0,0);

	split[0].x_min = 0;		split[0].x_max = (int)(haar[4].width);
	split[0].y_min = 0;		split[0].y_max = (int)(haar[4].height/3);

	split[1].x_min = 0;								split[1].x_max = (int)(haar[4].width);
	split[1].y_min = (int)((haar[4].width/3));		split[1].y_max = ((int)(haar[4].width/3)*2);

	split[2].x_min = 0;								split[2].x_max = (int)(haar[4].width);
	split[2].y_min = ((int)(haar[4].width/3)*2);	split[2].y_max = (int)(haar[4].height);

	m = mean(haar[4].data,haar[4].width*haar[4].height);
	feature_vec[0] = var(haar[0].data, m, haar[0].width*haar[0].height);
	
	feature_vec[1] = var_in_region(haar[1].data,split[0]);
	feature_vec[2] = var_in_region(haar[1].data,split[1]);
	feature_vec[3] = var_in_region(haar[1].data,split[2]);

	feature_vec[4] = var_in_region(haar[2].data,split[0]);
	feature_vec[5] = var_in_region(haar[2].data,split[1]);
	feature_vec[6] = var_in_region(haar[2].data,split[2]);

	feature_vec[7] = var_in_region(haar[3].data,split[0]);
	feature_vec[8] = var_in_region(haar[3].data,split[1]);
	feature_vec[9] = var_in_region(haar[3].data,split[2]);

	//next levlel
	//haar[4] = har_avg(img_reg,1,&haar[0],1);
	for (i = 1; i < scale; i++) {
	
		haar[1] = har_avg(img_reg,2,&haar[4],1);
		haar[2] = har_avg(img_reg,3,&haar[4],1);
		haar[3] = har_avg(img_reg,4,&haar[4],1);
	
		haar[4] = har_avg(img_reg,1,&haar[4],1);

		split[0].x_min = 0;		split[0].x_max = (int)(haar[4].width);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[4].height/3);

		split[1].x_min = 0;								split[1].x_max = (int)(haar[4].width);
		split[1].y_min = (int)((haar[4].width/3));		split[1].y_max = ((int)(haar[4].width/3)*2);

		split[2].x_min = 0;								split[2].x_max = (int)(haar[4].width);
		split[2].y_min = ((int)(haar[4].width/3)*2);	split[2].y_max = (int)(haar[4].height);

	 feature_vec[4 + ((i-1)*9)] = var_in_region(haar[1].data,split[0]);
	 feature_vec[5 + ((i-1)*9)] = var_in_region(haar[1].data,split[1]);
	 feature_vec[6 + ((i-1)*9)] = var_in_region(haar[1].data,split[2]);

	 feature_vec[7 + ((i-1)*9)] = var_in_region(haar[2].data,split[0]);
	 feature_vec[8 + ((i-1)*9)] = var_in_region(haar[2].data,split[1]);
	 feature_vec[9 + ((i-1)*9)] = var_in_region(haar[2].data,split[2]);
	 
	 feature_vec[10 + ((i-1)*9)] = var_in_region(haar[3].data,split[0]);
	 feature_vec[11 + ((i-1)*9)] = var_in_region(haar[3].data,split[1]);
	 feature_vec[12 + ((i-1)*9)] = var_in_region(haar[3].data,split[2]);

	 
	free(haar[0].data);
    free(haar[1].data);
    free(haar[2].data);
    free(haar[3].data);

	}

}

/*
 ------------------------------------------------------------
 Generates the feautre vector from a region
 currently a 2 scale, with hardcoded splits
*/
void feature_extractA(imageRGB *img_reg, double *feature_vec, int scale){
	trans_image haar[5];
	cBox split[4];
	double m;
	char str[50];
	int i;
	haar[0] = har_avg(img_reg,1,0,0);
    haar[1] = har_avg(img_reg,2,0,0);
    haar[2] = har_avg(img_reg,3,0,0);
    haar[3] = har_avg(img_reg,4,0,0);
	
	haar[4] = har_avg(img_reg,1,0,0);

	split[0].x_min = 0;		split[0].x_max = (int)(haar[0].width);
	split[0].y_min = 0;		split[0].y_max = (int)(haar[0].height);

	m = mean(haar[0].data,haar[0].width*haar[0].height);
	feature_vec[0] = var(haar[0].data, m, haar[0].width*haar[0].height);
	feature_vec[1] = var_in_region(haar[1].data,split[0]);
	feature_vec[2] = var_in_region(haar[2].data,split[0]);
	feature_vec[3] = var_in_region(haar[3].data,split[0]);

	//next levlel
	//haar[4] = har_avg(img_reg,1,&haar[0],1);
	for (i = 1; i < scale; i++) {
	
		haar[1] = har_avg(img_reg,2,&haar[4],1);
		haar[2] = har_avg(img_reg,3,&haar[4],1);
		haar[3] = har_avg(img_reg,4,&haar[4],1);
	
		haar[4] = har_avg(img_reg,1,&haar[4],1);

		split[0].x_min = 0;		split[0].x_max = (int)(haar[4].width);
		split[0].y_min = 0;		split[0].y_max = (int)(haar[4].height);

	 feature_vec[4 + ((i-1)*3)] = var_in_region(haar[1].data,split[0]);
	 feature_vec[5 + ((i-1)*3)] = var_in_region(haar[2].data,split[0]);
	 feature_vec[6 + ((i-1)*3)] = var_in_region(haar[3].data,split[0]);

	 
	free(haar[0].data);
    free(haar[1].data);
    free(haar[2].data);
    free(haar[3].data);

	}

}

void print_featurevec(double *featurevec) {
 int i;
 for (i = 0; i < FEATURE_SIZE; i++) {
	 printf("\n element (%d): %f",i, featurevec[i]);
 }

}


/*
 ------------------------------------------------------------
 adds a box to group, overlapping regions will be in same group
*/
void add_to_group(cbox_group *group,cBox *baseregion) {
	int i,w;
 
	for (i = 0; i < group_count+1;i++) {
		for(w = 0; w < group[i].count;w++) {

			if ((group[i].regions[w].x_min >= baseregion->x_min && group[i].regions[w].x_min <= baseregion->x_max) || (baseregion->x_min >= group[i].regions[w].x_min && baseregion->x_min <= group[i].regions[w].x_max)) {
            if ((group[i].regions[w].y_min >= baseregion->y_min && group[i].regions[w].y_min <= baseregion->y_max) || (baseregion->y_min >= group[i].regions[w].y_min && baseregion->y_min <= group[i].regions[w].y_max)) {
	
 					group[i].regions[group[i].count].x_max = baseregion->x_max;
					group[i].regions[group[i].count].x_min = baseregion->x_min;
					group[i].regions[group[i].count].y_max = baseregion->y_max;
					group[i].regions[group[i].count].y_min = baseregion->y_min;
					group[i].regions[group[i].count].face_distance = baseregion->face_distance/baseregion->volume;
				    group[i].count++;
					
					return;
			   } 
			 }
		}
		}	

		group[group_count].count = 1;
		group[group_count].regions = (cBox*) malloc(sizeof(cBox)*200);
		group[group_count].regions[0].x_max = baseregion->x_max;
		group[group_count].regions[0].x_min = baseregion->x_min;
		group[group_count].regions[0].y_max = baseregion->y_max;
		group[group_count].regions[0].y_min = baseregion->y_min;
		group[group_count].regions[0].face_distance = baseregion->face_distance/baseregion->volume;
		group_count++;
	

}

/*
 ------------------------------------------------------------
 Finds minimum face_distance in a pointer to regions
*/
cBox min_face_dist(cBox *reg,int n) {
int i,best_index;

double bestdist;
 for (i = 0; i < n;i++) {

	 if (reg[i].face_distance < bestdist || i ==0) {
		bestdist = reg[i].face_distance;
		best_index = i;
	 }
 }

 return (reg[best_index]);
}


/*
--------------------------------------------------
 Train feature vector used for face
*/
void train_feature(int file_number, int fsize,int scale) {
    
	printf("\nTraining avg feature vector (%d) images",file_number);

	double *tmpfeature = (double*) malloc(sizeof(double)*fsize);
	double *avgfeature = (double*) malloc(sizeof(double)*fsize);
	char str[50];
	imageRGB org;
	int i,j;

	//clear
	for (j = 0; j < fsize;j++) {
		tmpfeature[j] = avgfeature[j] = 0;
	}
	

	for (i = 0; i < file_number; i++) {

		   sprintf(str,"face_regions/small/%d.jpg",i);
		   org = read_JPEG_file(str);
		   feature_extractB(&org,tmpfeature,scale);

		   for (j = 0; j < fsize; j++) {
			   avgfeature[j] = ((avgfeature[j] + tmpfeature[j])/2);
		   }
		
		   free(org.data);
			printf("\n\t%d, done...",i);
	}
	
	//dispay the feature vector
	printf("\nAvg feature vector:\n");
	for (j = 0; j < fsize;j++) {
		printf("\n element (%d) (%f)",j,avgfeature[j]);
	}
	printf("\n\n");

}




/*---

*/
void load_recog() {
int i,j,k;
char str[50];
face face_db[40];
double *featurevec = (double*) malloc(sizeof(double)*FEATURE_SIZE);
double *tmp_vec = (double*) malloc(sizeof(double)*FEATURE_SIZE);
imageRGB img;


//training the faces..
 for (i = 0; i < 40; i++) {
	 face_db[i].feature_vec = (double*) malloc(sizeof(double)*FEATURE_SIZE);
	
	 for (j = 0; j < 3; j++) {
		sprintf(str,"faces/s%d/%d.jpg",i+1,j+1);
		img = read_JPEG_file (str);
		feature_extractC(&img, tmp_vec,3);

		if (j != 0) {
			for (k = 0; k < FEATURE_SIZE;k++) {
				face_db[i].feature_vec[k] = (face_db[i].feature_vec[k] + tmp_vec[k])/2;
			}
		} else {
			for (k = 0; k < FEATURE_SIZE;k++) {
				face_db[i].feature_vec[k] = tmp_vec[k];
			}
		}

		free (img.data);
	 }
	 face_db[i].facenr = i;
	 

	 printf("\n chunk %d stored,", i+1);
	 //print_featurevec(face_db[i].feature_vec);

 }



 int best_ind;
 double best_dist;
 double dist;
 int fail, succ;
 fail = succ = 0;

 for (i = 0; i < 40; i++) {

	 sprintf(str,"faces/s%d/1.jpg",i+1);
	 img = read_JPEG_file (str);

	 feature_extractC(&img, featurevec,3);
	 
	 //find the nearest feature vec.
	 for (j = 0; j < 40; j++) {

		dist = dist_bhatta2(featurevec, face_db[j].feature_vec);
		//dist = dist_leastsq2(featurevec, face_db[j].feature_vec);
		
		if (dist < best_dist || j == 0) {
			best_dist = dist;
			best_ind = j;
		}
	 }

	 printf("\n %d vs %d",i,best_ind);

	 if (best_ind == face_db[i].facenr) {
		//printf("\n SUCCESS",best_dist);
		succ++;
	 } else {
		 fail++;
		// printf("\n FAILURE",i,best_ind,best_dist);
	 }
 }

 	 printf("\n ");
	 printf("Success rate: %d, failure rate: %d", succ,fail);
 	 printf("\n\n");
}



/*
 ------------------------------------------------------------
 Main
*/
int main (int argc, char** argv) {
	int   quality = 75; // in range 0-100  
  	char* file_in = argv[1];
  	char* file_out= argv[2];
	cbox_group *grp = (cbox_group*) malloc(sizeof(cbox_group)*100);

	imageRGB img;
	imageRGB org;
	
/* 
  -> 2 scale mode C <-
	facevec[0] = 6587;
	facevec[1] = 426;
	facevec[2] = 500;
	facevec[3] = 1713;
	facevec[4] = 1700;;
	facevec[5] = 119;
	facevec[6] = 130;
	facevec[7] = 325;
	facevec[8] = 328;
	facevec[9] = 92;
	facevec[10] = 103;
	facevec[11] = 329;
	facevec[12] = 329;
	facevec[13] = 1761;
	facevec[14] = 2788;
	facevec[15] = 5820;
	facevec[16] = 5660;
	facevec[17] = 799;
	facevec[18] = 825;
	facevec[19] = 1310;
	facevec[20] = 1461;
	facevec[21] = 376;
	facevec[22] = 440;
	facevec[23] = 908;
	facevec[24] = 882;
	facevec[25] = 9850;
	facevec[26] = 15782;
	facevec[27] = 23765;
	facevec[28] = 19261;
	facevec[29] = 5611;
	facevec[30] = 5971;
	facevec[31] = 7876;
	facevec[32] = 7256;
	facevec[33] = 1800;
	facevec[34] = 2008;
	facevec[35] = 4206;
	facevec[36] = 3492;


	facevec[0] = 10779;
	facevec[1] = 426;
	facevec[2] = 500;
	facevec[3] = 1713;
	facevec[4] = 1700;;
	facevec[5] = 119;
	facevec[6] = 130;
	facevec[7] = 325;
	facevec[8] = 328;
	facevec[9] = 92;
	facevec[10] = 103;
	facevec[11] = 329;
	facevec[12] = 329;
	facevec[13] = 1761;
	facevec[14] = 2788;
	facevec[15] = 5820;
	facevec[16] = 5660;
	facevec[17] = 799;
	facevec[18] = 825;
	facevec[19] = 1310;
	facevec[20] = 1461;
	facevec[21] = 376;
	facevec[22] = 440;
	facevec[23] = 908;
	facevec[24] = 882;

	->    featureA 2scale      <-
	facevec[0] = 6525;
	facevec[1] = 1309;
	facevec[2] = 278;
	facevec[3] = 253;
	facevec[4] = 3586;;
	facevec[5] = 2810;
	facevec[6] = 2451;

	->    featureA 3scale      <-
	facevec[0] = 6522;
	facevec[1] = 1309;
	facevec[2] = 278;
	facevec[3] = 253;
	facevec[4] = 3586;;
	facevec[5] = 2810;
	facevec[6] = 2451;
	facevec[7] = 30619;;
	facevec[8] = 5930;
	facevec[9] = 2763;

->    featureB 2scale      <-

	facevec[0] = 6473;
	facevec[1] = 981;
	facevec[2] = 1827;
	facevec[3] = 1385;
	facevec[4] = 3263;
	facevec[5] = 5690;
	facevec[6] = 3154;
	facevec[7] = 900;
	facevec[8] = 1433;
	facevec[9] = 3886;
	facevec[10] = 516;
	facevec[11] = 1070;
	facevec[12] = 3551;

	->    featureB 3scale      <-
	facevec[0] = 6525;
	facevec[1] = 981;
	facevec[2] = 1827;
	facevec[3] = 1428;
	facevec[4] = 3263;
	facevec[5] = 5690;
	facevec[6] = 3154;
	facevec[7] = 900;
	facevec[8] = 1433;
	facevec[9] = 3886;
	facevec[10] = 516;
	facevec[11] = 1070;
	facevec[12] = 3551;
	facevec[13] = 13757;
	facevec[14] = 12052;
	facevec[15] = 37909;
	facevec[16] = 5864;
	facevec[17] = 7064;
	facevec[18] = 5555;
	facevec[19] = 2821;
	facevec[20] = 4352;
	facevec[21] = 2310;

*/

	facevec[0] = 6525;
	facevec[1] = 981;
	facevec[2] = 1827;
	facevec[3] = 1428;
	facevec[4] = 3263;
	facevec[5] = 5690;
	facevec[6] = 3154;
	facevec[7] = 900;
	facevec[8] = 1433;
	facevec[9] = 3886;
	facevec[10] = 516;
	facevec[11] = 1070;
	facevec[12] = 3551;
	facevec[13] = 13757;
	facevec[14] = 12052;
	facevec[15] = 37909;
	facevec[16] = 5864;
	facevec[17] = 7064;
	facevec[18] = 5555;
	facevec[19] = 2821;
	facevec[20] = 4352;
	facevec[21] = 2310;


    
	double m;

FEATURE_SIZE= 37;
double *featurevec = (double*) malloc(sizeof(double)*FEATURE_SIZE);
	


// face recognition
			
		load_recog();
		return 0;
	
// end
	

       cPixel *pixeldata;

       cBox *c1_region;
       cBox *c2_region;
       cBox *c3_region;
       cBox *c4_region;


 
	if (argc != 3) {
		printf("ERROR: wrong number of arguments.\n");
		printf("Usage: ./test_app <input_img.jpg> <output_img.jpg>\n");
		return -1;
	}

	cPixel drawcol;
	int w;

	// decode/load image into memory
	img = read_JPEG_file (file_in);
	org = read_JPEG_file (file_in);

	printf("\n");


//train 

    train_feature(16,FEATURE_SIZE,3);
	feature_extractB(&org,featurevec,3);
	return 0;


 //end


	treshSkin(&img);
	write_JPEG_file("tresh_result.jpg",quality,img);

//region merging
	c1_region = generateRegion(&img,&org);
	c2_region = mergeRegions(c1_region,&c2_count);
	c3_region = mergeRegions(c2_region,&c3_count);
	//c3_region = merge_regions(c1_region,c2_region,count_regions,c2_count,&c3_count);
	c4_count = 0;
	//c4_region = mergeRegions(c3_region,&c4_count);
	//	c4_region = merge_regions(c2_region,c3_region,c2_count,c3_count,&c4_count);

	cBox *boxfinal = (cBox*) malloc(sizeof(cBox)* (count_regions + c2_count + c3_count + c4_count));

	int reject = 0;
	 for (w = 0; w < count_regions;w++) {
	   if (add_region(boxfinal,c1_region[w]) == 0) {reject++;}
	 }
	 for (w = 0; w < c2_count;w++) {
	  if ( add_region(boxfinal,c2_region[w]) == 0) {reject++;}
	 }
	 for (w = 0; w < c3_count;w++) {
	   if (add_region(boxfinal,c3_region[w]) == 0) {reject++;}
	 }
	 for (w = 0; w < c4_count;w++) {
	   if (add_region(boxfinal,c4_region[w]) == 0) {reject++;}
	 }
	 //free the temporary regions, all stored inn boxfinal
	 free(c1_region);
	 free(c2_region);
	 free(c3_region);
	 free(c4_region);
	 printf("\n\nFinal number of regions: %d (%d discarded)\n",count_final,reject);

	 //region plotting
	 /*
	 drawcol.red = 255;drawcol.green = 0;drawcol.blue = 0;
	 for (w = 0; w < count_regions;w++) {
	    plot_box(&org,&c1_region[w],drawcol);
	 }


         drawcol.red = 255;drawcol.green = 0;drawcol.blue = 255;
	 for (w = 0; w < c2_count;w++) {
	   // plot_box(&org,&c2_region[w],drawcol);
	 }
	 
         drawcol.red = 255;drawcol.green = 255;drawcol.blue = 255;
	 for (w = 0; w < c3_count;w++) {
	  // plot_box(&org,&c3_region[w],drawcol);
	 } 

	 drawcol.red = 255;drawcol.green = 255;drawcol.blue = 255;
	 for (w = 0; w < count_final; w++) {
	   plot_box(&org,&boxfinal[w],drawcol);
	 }

   	 write_JPEG_file("region_final.jpg",quality,org);
	 //return 0;
	 */
     char str[50];
	 imageRGB reg_img;

//wavelet decomposition and featurevector creation
     count_regions = 0;
     // c1_region = (cBox*) malloc(sizeof(cBox)*count_final);

     for (w = 0; w < count_final;w++) {
           reg_img = image_from_region(boxfinal[w],&org);
           sprintf(str,"output/reg%d.jpg",w);
           write_JPEG_file(str,quality,reg_img);
	   
	   printf("\n");
	   feature_extractB(&reg_img,featurevec,3);


	   boxfinal[w].face_distance = dist_bhatta(featurevec);
	   //boxfinal[w].face_distance = dist_leastsq(featurevec);

	   printf("\t%d/%d complete, distance: %f",w,count_final-1,boxfinal[w].face_distance);
           free(reg_img.data);

	 }
	 
	 drawcol.red = 0;drawcol.green = 255;drawcol.blue = 255;
	
	 //categorise accepted face regions and discard accepted regions with overlap 
	 for (w = 0; w < count_final; w++) {
		 if (boxfinal[w].face_distance < DIST_TRESHOLD) {
		     add_to_group(grp,&boxfinal[w]);
		 }

	 }

	 printf("\nNumber of groups: %d",group_count);

	 drawcol.red = 0;drawcol.green = 255;drawcol.blue = 255;
	 cBox minbox;
	 for (w = 0; w < group_count; w++) {
	   printf("\n\t\tgroup: %d, number of regions in group: %d",w,grp[w].count);
		 minbox = min_face_dist(grp[w].regions, grp[w].count);
		 printf("\n\t\t %d, %d, %d, %d",minbox.x_min, minbox.y_min, minbox.x_max, minbox.y_max);
		 minbox.volume = 1;
		 plot_box(&org,&minbox, drawcol); 
	} 

	 write_JPEG_file(file_out,quality,org);

	free(sintbl);
	free(costbl);
	free(sintblh);
	free(costblh);
	free(pixeldata);
	free(featurevec);
	free(grp);
	//free(original);
    printf("\n");

	return 0;
}

